//
// Copyright © 2024 MediaMall Technologies, Inc.  All rights reserved.
//

#ifndef YCSConnect_h
#define YCSConnect_h

#import <Foundation/Foundation.h>
#import <YounifyConnectSDK/Logging.h>
#import <YounifyConnectSDK/Tokens.h>
#import <YounifyConnectSDK/Cancellation.h>

/// Younify Connect SDK startup options.
///
/// Set the properties of this object to configure the sdk.
/// The ``key`` and ``tokenDelegate`` properties are required.
/// The ``accessToken`` and ``refreshToken`` properties are required for linking and fetching service content for the user and can be set at a later time if unknown at startup. The tokens can be created from the backend api.
/// The ``tokenDelegate`` must be set to handle updated tokens and requests from the sdk to renew the tokens from the backend api.
@interface YCSConnectOptions : NSObject

/// Required key.
@property (nonatomic, copy) NSString *key;
/// Optional log delegate for receiving log events.
@property (nonatomic, strong) id<YCSLogDelegate> logDelegate;
/// Optional token delegate for intercepting token events and renewals.
@property (nonatomic, strong) id<YCSTokenDelegate> tokenDelegate;
/// The user's access token.
@property (nonatomic, copy) NSString *accessToken;
/// The user's refresh token.
@property (nonatomic, copy) NSString *refreshToken;
/// Extra options.
@property (nonatomic, copy) NSDictionary<NSString *, id> *extra;

@end


/// Streaming service link.
///
/// Contains informative user properties about the linked streaming service.
@interface YCSStreamingServiceLink : NSObject

/// Service username or email.
@property (nonatomic, copy) NSString *username;
/// Selected profile name if the service has profiles.
@property (nonatomic, copy) NSString *profileName;
/// Selected profile image if the service has profiles.
@property (nonatomic, copy) NSString *profileImageUrl;
/// Whether this profile is pin protected if the service has profiles.
@property (nonatomic, assign) BOOL isProfilePINProtected;
/// Whether this link is broken or otherwise has a problem that requires management or unlinking to address.
@property (nonatomic, assign) BOOL isBroken;

@end


/// Streaming service.
///
/// Represents a streaming service.
@interface YCSStreamingService : NSObject

/// Streaming service name.
@property (nonatomic, copy) NSString *name;
/// Streaming service small thumbnail image url.
@property (nonatomic, copy) NSString *smallThumbnailUrl;
/// Streaming service large thumbnail image url.
@property (nonatomic, copy) NSString *largeThumbnailUrl;
/// Whether this streaming service is currently available or is undergoing maintenance.
@property (nonatomic, assign) BOOL isAvailable;
/// Streaming service link info if this service was linked by the user.
@property (nonatomic, strong) YCSStreamingServiceLink *link;

@end


/// Streaming service content type.
///
/// The type of content the ``YCSStreamingContent`` object represents.
typedef NS_ENUM(NSInteger, YCSStreamingContentType)
{
    /// Invalid type.
    YCSStreamingContentTypeInvalid = -1,
    /// Folder.
    YCSStreamingContentTypeFolder,
    /// Video.
    YCSStreamingContentTypeVideo
};


/// Streaming service content.
///
/// Represents a content item from a streaming service.
@interface YCSStreamingContent : NSObject

/// Content type.
@property (nonatomic, assign) YCSStreamingContentType type;
/// Title.
@property (nonatomic, copy) NSString *title;
/// Overview, synopsis or description.
@property (nonatomic, copy) NSString *overview;
/// Duration.
@property (nonatomic, assign) long long duration;
/// Air date.
@property (nonatomic, copy) NSDate *airDate;
/// Small thumbnail image url.
@property (nonatomic, copy) NSString *smallThumbnailUrl;
/// Large thumbnail image url.
@property (nonatomic, copy) NSString *largeThumbnailUrl;
/// Streaming service specific content id for this item.
@property (nonatomic, copy) NSString *itemID;
/// Series title.
@property (nonatomic, copy) NSString *series;
/// Series season.
@property (nonatomic, copy) NSString *season;
/// Series episode.
@property (nonatomic, copy) NSString *episode;
/// Release year.
@property (nonatomic, copy) NSString *releaseYear;
/// Content rating.
@property (nonatomic, copy) NSString *contentRating;
/// Streaming service specific watch now url.
@property (nonatomic, copy) NSString *watchNowUrl;

@end


/// Streaming service content details.
///
/// Holds additional content item details for a ``YCSStreamingContent`` object.
@interface YCSStreamingContentDetails : NSObject

/// Browse path of this item.
@property (nonatomic, copy) NSString *browsePath;
/// Estimated duration of this item including advertisements.
@property (nonatomic, assign) long long estimatedDuration;

@end


/// Streaming service content category.
///
/// Represents a streaming service category for a user.
@interface YCSStreamingCategory : NSObject

/// Category name.
@property (nonatomic, copy) NSString *name;

@end


/// Well-known streaming service content categories.
///
/// Holds a list of well-known streaming service categories.
@interface YCSStreamingCategories

/// Your Watchlist.
+ (YCSStreamingCategory *)watchlist;
/// Continue Watching.
+ (YCSStreamingCategory *)continueWatching;
/// Recommended for You.
+ (YCSStreamingCategory *)recommendedForYou;
/// Trending &amp; Popular.
+ (YCSStreamingCategory *)trendingAndPopular;
/// Critically Acclaimed.
+ (YCSStreamingCategory *)criticallyAcclaimed;

@end


/// Fetch content category service result.
///
/// Holds the results of a fetch content request for a given streaming service.
@interface YCSFetchContentCategoryServiceResult : NSObject

/// Streaming service the content items belong to.
@property (nonatomic, strong) YCSStreamingService *service;
/// Content items for `service`.
@property (nonatomic, strong) NSArray<YCSStreamingContent *> *content;

@end


/// Fetch content category result.
///
/// Holds the results of a fetch content request for a category and each streaming service.
@interface YCSFetchContentCategoryResult : NSObject

/// Streaming category the results apply to.
@property (nonatomic, strong) YCSStreamingCategory *category;
/// Service results for the specified category.
@property (nonatomic, strong) NSArray<YCSFetchContentCategoryServiceResult *> *services;

@end


/// Younify Connect SDK main object.
///
/// Use the ``shared`` singleton instance to interact with the sdk. Before the sdk can be meaningfully used ``startWithOptions:`` must be called.
@interface YCSConnect : NSObject

/// The shared singleton instance.
+ (instancetype)shared;

/// Starts the sdk with the specified options.
/// - Parameter options: The start options.
///
/// Before using the sdk it must be started by using this method with a valid ``YCSConnectOptions`` instance. It is safe to call this method repeatedly to change the options dynamically at runtime if desired.
- (void)startWithOptions:(YCSConnectOptions *)options;

/// Set the user's tokens.
/// - Parameters:
///   - accessToken: The user's access token.
///   - refreshToken: The user's refresh token.
///
/// Set the user's access and refresh tokens that are required for accessing streaming services. User tokens can be obtained by creating a user object from the backend api.
- (void)setUserTokens:(NSString *)accessToken refreshToken:(NSString *)refreshToken;


/// Link a streaming service.
/// - Parameters:
///   - service: The service object to link.
///   - completionHandler: The completion handler to call when the link service request is complete.
///
/// Opens the service linking flow which involves a web view login for the specified service followed by profile selection and optional pin collection if supported. `completionHandler` will be called with the link result.
/// If linking is successful, the success flag will be TRUE and error nil. If linking fails the success flag will be FALSE and error non-nil. If linking was canceled then the success flag will be FALSE and error nil.
- (void)linkService:(YCSStreamingService *)service completionHandler:(void (^)(YCSStreamingService *service, BOOL success, NSError *error))completionHandler;

/// Manage a linked service.
/// - Parameters:
///   - service: The service object to manage.
///   - completionHandler: The completion handler to call when the manage service request is complete.
///
/// Opens the linked service management flow which allows the user to change their profile selection and pin if supported. `completionHandler` will be called with the link management result.
- (void)manageLinkedService:(YCSStreamingService *)service completionHandler:(void (^)(YCSStreamingService *service, NSError *error))completionHandler;

/// Unlink a linked service.
/// - Parameters:
///   - service: The service object to unlink.
///   - completionHandler: The completion handler to call when the unlink service request is complete.
///
/// Unlinks a linked service and will call `completionHandler` with the result.
- (void)unlinkService:(YCSStreamingService *)service completionHandler:(void (^)(YCSStreamingService *service, NSError *error))completionHandler;

/// Fetch all available services.
/// - Parameter completionHandler: The completion handler to call when the fetch services request is complete.
///
/// Retrieves all available services and will call `completionHandler` with the result.
- (void)fetchServices:(void (^)(NSArray<YCSStreamingService *> *services, NSError *error))completionHandler;

/// Fetch linked services.
/// - Parameter completionHandler: The completion handler to call when the fetch linked services request is complete.
///
/// Retrieves a list of all the user's linked services and will call `completionHandler` with the result.
- (void)fetchLinkedServices:(void (^)(NSArray<YCSStreamingService *> *services, NSError *error))completionHandler;

/// Fetch broken services.
/// - Parameter completionHandler: The completion handler to call when the fetch broken services request is complete.
///
/// Retrieves a list of all the user's broken services and will call `completionHandler` with the result.
- (void)fetchBrokenServices:(void (^)(NSArray<YCSStreamingService *> *services, NSError *error))completionHandler;

/// Fetch all streaming categories.
/// - Parameter completionHandler: The completion handler to call when the fetch categories request is complete.
///
/// Retrives all available streaming service categories and will call `completionHandler` with the result.
- (void)fetchCategories:(void (^)(NSArray<YCSStreamingCategory *> *categories, NSError *error))completionHandler;

/// Fetch streaming service category contents.
/// - Parameters:
///   - categories: The list of categories to fetch content for.
///   - services: The list of streaming services to fetch content for.
///   - serviceErrorHandler: The optional error handler to call when a problem occurs accessing a service.
///   - categoryServiceCompletionHandler: The optional completion handler to call when retrieving content for a single category and service.
///   - allCompletionHandler: The optional completion handler to call when all content has been retrieved.
///   - cancellationTicket: The ticket to manually cancel the operation.
///
/// Retrieves the content for the `categories` and `services` specified. `serviceErrorHandler` will be called when service error occurred. `categoryServiceCompletionHandler` will be called with the results for a single category-service pair while `allCompletionHandler` will be called with all of the results.
- (void)fetchContent:(NSArray<YCSStreamingCategory *> *)categories
        services:(NSArray<YCSStreamingService *> *)services
        serviceErrorHandler:(void (^)(YCSStreamingService *service, NSError *error))serviceErrorHandler
        categoryServiceCompletionHandler:(void (^)(YCSStreamingCategory *category, YCSStreamingService *service, NSArray<YCSStreamingContent *> *content, NSError *error))categoryServiceCompletionHandler
        allCompletionHandler:(void (^)(NSArray<YCSFetchContentCategoryResult *> *result, NSError *error))allCompletionHandler
        cancellationTicket:(YCSCancellationTicket *)cancellationTicket;

/// Fetch content children.
/// - Parameters:
///   - content: The parent content item to fetch children for.
///   - completionHandler: The completion handler to call when the fetch children request is complete.
///
/// Retrives the children of the given parent `content` item and will call `completionHandler` with the results. The `content` object must be a folder to have children.
///
/// Examples include using a `content` that represents a series where the children would be its seasons, or a `content` that represents a season where the children would be its episodes.
- (void)fetchContentChildren:(YCSStreamingContent *)content completionHandler:(void (^)(YCSStreamingContent *content, NSArray<YCSStreamingContent *> *children, NSError *error))completionHandler;

/// Fetch content details.
/// - Parameters:
///   - content: The content item to retrieve details for.
///   - completionHandler: The completion handler to call when the fetch content details request is complete.
///
/// Retrieves `content` details and will call `completionHandler` with the results.
///
/// Details or additional metadata properties include series, season, episode, releaseYear and contentRating.
- (void)fetchContentDetails:(YCSStreamingContent *)content completionHandler:(void (^)(YCSStreamingContent *content, YCSStreamingContentDetails *details, NSError *error))completionHandler;

/// Search a streaming service.
/// - Parameters:
///   - service: The streaming service to search.
///   - searchPhrase: The search terms.
///   - completionHandler: The completion handler to call when the search request is complete.
///
/// Searches the given `service` with the specified `searchPhrase` and will call `completionHandler` with the results.
- (void)searchService:(YCSStreamingService *)service searchPhrase:(NSString *)searchPhrase completionHandler:(void (^)(YCSStreamingService *service, NSArray<YCSStreamingContent *> *results, NSError *error))completionHandler;

/// Fetch an image.
/// - Parameters:
///   - url: The image url to fetch.
///   - completionHandler: The completion handler to call when the fetch image request is complete.
///
/// Retrieves the image from the specified image `url` and will call `completionHandler` with the image data result. 
///
/// Directly fetching image urls in any other way can lead to missing data or 404s depending on the streaming service.
- (void)fetchImage:(NSString *)url completionHandler:(void (^)(NSData *image, NSError *error))completionHandler;

@end

#endif /* YCSConnect_h */
